#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Save changes.xml for the firmware version to be updated."""


import sys
import os
import re

from iobase.xmldata import (XmlConf,)
from ionas import (request,)
from fal_info import (FAL_SYSINFO_PATH, ReqTypes)


ENCODING = "UTF-8"

SYSINFO_NAME = "sysinfo"

FWVERSION_PATH = "fwversion"
FWVERSION_RE = "([0-9]+.[0-9]+).*"
CHANGE_NAME = "change"
VERSION_ATTR = "version"
MIN_VERSION_UNIT = 0.01


def __send_request(target, type, body="", file_path=""):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    if file_path:
        entry.set_file(file_path)
    return request.Request(target).request(entry).get_entry()


def __load_xml_conf(xml_path):
    """Return "XmlConf" instance loding the "xml_path"."""

    return XmlConf(xml_path, encoding=ENCODING)


def __save_xml_conf(xml_conf, xml_path):
    """Save "xml_conf" to the "xml_path"."""

    tmp_path = xml_path + "." + str(os.getpid())
    xml_conf.save(tmp_path)
    os.rename(tmp_path, xml_path)


def __to_float_version(version_str):
    """Return the version number which type is a float."""

    return float(re.match(FWVERSION_RE, version_str).group(1))


def get_current_version():
    """Return the current firmware version."""

    sysinfo_info = __send_request(FAL_SYSINFO_PATH, ReqTypes.QUERY,
                                    SYSINFO_NAME).get()
    version_str = sysinfo_info.get_node(FWVERSION_PATH).get_text()
    return __to_float_version(version_str)


def get_base_version(changes_xml_path):
    """Return the base firmware version for changes."""

    base_version = get_current_version()
    try:
        changes_xml_conf = __load_xml_conf(changes_xml_path)
        for changes in changes_xml_conf.get_node_list(CHANGE_NAME):
            change_version = __to_float_version(
                                        changes.get_attr_value(VERSION_ATTR))
            if base_version >= change_version:
                base_version = change_version - MIN_VERSION_UNIT
    except:
        # Ignore error
        pass

    return base_version


def update_changes(changes_xml, base_version):
    """Update "changes_xml_conf" data ""."""

    idx = 0
    for changes in changes_xml.get_node_list(CHANGE_NAME):
        try:
            change_version = __to_float_version(
                                        changes.get_attr_value(VERSION_ATTR))
            if change_version <= base_version:
                changes_xml.delete_child(CHANGE_NAME, index=idx)
            else:
                idx += 1
        except:
            # Ignore error
            idx += 1


#
# Main
#

if len(sys.argv) < 3:
    print "Error: too few arguments!!"
    print
    print "usage : " + sys.argv[0] + " CHANGES_BASE_XML CHANGES_XML"
    sys.exit(1)

try:
    changes_xml_conf = __load_xml_conf(sys.argv[1])
    base_version = get_base_version(sys.argv[2])
    update_changes(changes_xml_conf.get_root_node(), base_version)
    __save_xml_conf(changes_xml_conf, sys.argv[2])

except:
    sys.exit(2)
